from decimal import Decimal

from ypricemagic.utils.contracts import Contract
from ypricemagic.utils.raw_calls import _totalSupplyReadable


class ERC20:
    def __init__(self, token_address) -> None:
        self.address = token_address
        self.contract = Contract(self.address)
        self.build_name = self.contract.__dict__['_build']['contractName']

    def total_supply(self, block=None) -> Decimal:
        return _totalSupplyReadable(self.address, block)




import logging

from brownie import chain
from ypricemagic.networks import Network
from ypricemagic.price_modules.balancer.v1.v1 import BalancerV1
from ypricemagic.price_modules.balancer.v2.v2 import BalancerV2

logger = logging.getLogger(__name__)


class Balancer:
    def __init__(self) -> None:
        self.versions = []
        try:
            self.v1 = BalancerV1()
            self.versions.append(self.v1)
        except ImportError:
            pass
        try: 
            self.v2 = BalancerV2()
            self.versions.append(self.v2)
        except ImportError:
            pass

    #@memory.cache()
    def is_balancer_pool(self, token_address: str) -> bool:
        return any(v.is_pool(token_address) for v in self.versions)
    
    def get_pool_price(self, token_address: str, block=None):
        if self.v1.is_pool(token_address): return self.v1.get_pool_price(token_address, block)
        if self.v2.is_pool(token_address): return self.v2.get_pool_price(token_address, block)

    def get_price(self, token_address: str, block=None):
        if self.is_balancer_pool(token_address): return self.get_pool_price(token_address, block=block)

        price = None    
        
        if ( # NOTE: Only query v2 if block queried > v2 deploy block plus some extra blocks to build up liquidity
            (chain.id == Network.Mainnet and (not block or block > 12272146 + 100000))
            or (chain.id == Network.Fantom and (not block or block > 16896080))
            ): 
            price = self.v2.get_token_price(token_address, block)
            if price:
                logger.debug(f"balancer v2 -> ${price}")
                return price

        if not price and chain.id == Network.Mainnet:      
            price = self.v1.get_token_price(token_address, block)
            if price:
                logger.debug(f"balancer v1 -> ${price}")
                return price
        

balancer = Balancer()



import ypricemagic.magic
from brownie import multicall
from ypricemagic.utils.contracts import Contract
from ypricemagic.utils.raw_calls import _decimals, _totalSupplyReadable, raw_call


def is_gelato_pool(token_address: str) -> bool:
    pool = Contract(token_address)
    return hasattr(pool, 'gelatoBalance0') and hasattr(pool, 'gelatoBalance1')

def get_price(token_address: str, block=None):
    token0 = raw_call(token_address,'token0()',block=block,output='address')
    token1 = raw_call(token_address,'token1()',block=block,output='address')
    balance0 = raw_call(token_address,'gelatoBalance0()',block=block,output='int')
    balance1 = raw_call(token_address,'gelatoBalance1()',block=block,output='int')
    balance0 /= 10 ** _decimals(token0,block)
    balance1 /= 10 ** _decimals(token1,block)
    totalSupply = _totalSupplyReadable(token_address,block)
    totalVal = balance0 * ypricemagic.magic.get_price(token0,block) + balance1 * ypricemagic.magic.get_price(token1,block)
    return totalVal / totalSupply



import os

from brownie import network
from ypricemagic import magic
from ypricemagic.constants import dai, usdc, wbtc, weth
from ypricemagic.exceptions import *
from ypricemagic.magic import get_price, get_prices
from ypricemagic.networks import Network
from ypricemagic.utils.contracts import Contract
from ypricemagic.utils.multicall import fetch_multicall
from ypricemagic.utils.raw_calls import _balanceOf as balanceOf
from ypricemagic.utils.raw_calls import _balanceOfReadable as balanceOfReadable
from ypricemagic.utils.raw_calls import _symbol as symbol
from ypricemagic.utils.raw_calls import raw_call

from y.erc20 import decimals, totalSupply, totalSupplyReadable

__all__ = [
    ### you can reach the below functions, classes, and variables using ###
    ###  `y.__name__` or `from y import __name__`.  ###

    # contract stuff
    'Contract',

    # network stuff
    'Network',

    # magic
    'magic'

    # prices
    'get_price',
    'get_prices',

    # constants
    'weth',
    'dai',
    'usdc',
    'wbtc',

    # multicall
    'fetch_multicall',

    # raw calls
    'decimals',
    'symbol',
    'balanceOf',
    'balanceOfReadable',
    'totalSupply',
    'totalSupplyReadable',
    'raw_call'
]

if not network.is_connected():
    try:
        network_name = os.environ['BROWNIE_NETWORK_ID']
    except KeyError:
        raise KeyError('In order to use pricemagic outside of a brownie project directory, you will need to set $BROWNIE_NETWORK_ID environment variable with the id of your preferred brownie network connection.')
    network.connect(network_name)


